//
//  NSAttributedString.m
//  FoundationExtension
//
//  Created by Jeong YunWon on 12. 10. 31..
//  Copyright (c) 2012 youknowone.org. All rights reserved.
//

#import "NSAttributedString.h"

#import "debug.h"

static NSArray *__NSWritingDirectionAttributeArrayFromAttributes(NSWritingDirectionAttributes writingDirection) {
    NSMutableArray *array = [NSMutableArray array];
    if (writingDirection & NSWritingDirectionAttributeLRE) {
        const NSWritingDirectionAttributes LRE = NSWritingDirectionLeftToRight | NSTextWritingDirectionEmbedding;
        dassert(LRE == 0);
        [array addObject:[NSNumber numberWithUnsignedInteger:LRE]];
    }
    if (writingDirection & NSWritingDirectionAttributeRLE) {
        const NSWritingDirectionAttributes RLE = NSWritingDirectionRightToLeft | NSTextWritingDirectionEmbedding;
        dassert(RLE == 1);
        [array addObject:[NSNumber numberWithUnsignedInteger:RLE]];
    }
    if (writingDirection & NSWritingDirectionAttributeLRO) {
        const NSWritingDirectionAttributes LRO = NSWritingDirectionLeftToRight | NSTextWritingDirectionOverride;
        dassert(LRO == 2);
        [array addObject:[NSNumber numberWithUnsignedInteger:LRO]];
    }
    if (writingDirection & NSWritingDirectionAttributeRLO) {
        const NSWritingDirectionAttributes RLO = NSWritingDirectionRightToLeft | NSTextWritingDirectionOverride;
        dassert(RLO == 3);
        [array addObject:[NSNumber numberWithUnsignedInteger:RLO]];
    }
    return array;
}

@implementation NSMutableAttributedString (AttributeProperties)

- (void)addAttributeWritingDirectionArray:(NSArray *)writingDirectionArray range:(NSRange)range {
    [self addAttribute:NSWritingDirectionAttributeName value:writingDirectionArray range:range];
}

- (void)addAttributeWritingDirection:(NSWritingDirectionAttributes)writingDirection range:(NSRange)range {
    [self addAttribute:NSWritingDirectionAttributeName value:__NSWritingDirectionAttributeArrayFromAttributes(writingDirection) range:range];
}

- (void)removeAttributeWritingDirectionFromRange:(NSRange)range {
    [self removeAttribute:NSWritingDirectionAttributeName range:range];
}

//! Generated by NSAttributedStringCategoryImplementation.py
- (void)addAttributeFont:(NSFont *)font range:(NSRange)range { [self addAttribute:NSFontAttributeName value:font range:range]; }
//! Generated by NSAttributedStringCategoryImplementation.py
- (void)removeAttributeFontFromRange:(NSRange)range { [self removeAttribute:NSFontAttributeName range:range]; }


//! Generated by NSAttributedStringCategoryImplementation.py
- (void)addAttributeParagraphStyle:(NSParagraphStyle *)paragraphStyle range:(NSRange)range { [self addAttribute:NSParagraphStyleAttributeName value:paragraphStyle range:range]; }
//! Generated by NSAttributedStringCategoryImplementation.py
- (void)removeAttributeParagraphStyleFromRange:(NSRange)range { [self removeAttribute:NSParagraphStyleAttributeName range:range]; }


//! Generated by NSAttributedStringCategoryImplementation.py
- (void)addAttributeForegroundColor:(NSColor *)foregroundColor range:(NSRange)range { [self addAttribute:NSForegroundColorAttributeName value:foregroundColor range:range]; }
//! Generated by NSAttributedStringCategoryImplementation.py
- (void)removeAttributeForegroundColorFromRange:(NSRange)range { [self removeAttribute:NSForegroundColorAttributeName range:range]; }


//! Generated by NSAttributedStringCategoryImplementation.py
- (void)addAttributeUnderlineStyle:(NSUnderlineStyle)underlineStyle range:(NSRange)range { [self addAttribute:NSUnderlineStyleAttributeName value:[NSNumber numberWithUnsignedInteger:underlineStyle] range:range]; }
//! Generated by NSAttributedStringCategoryImplementation.py
- (void)removeAttributeUnderlineStyleFromRange:(NSRange)range { [self removeAttribute:NSUnderlineStyleAttributeName range:range]; }


//! Generated by NSAttributedStringCategoryImplementation.py
- (void)addAttributeSuperscript:(BOOL)superscript range:(NSRange)range { [self addAttribute:NSSuperscriptAttributeName value:[NSNumber numberWithBool:superscript] range:range]; }
//! Generated by NSAttributedStringCategoryImplementation.py
- (void)removeAttributeSuperscriptFromRange:(NSRange)range { [self removeAttribute:NSSuperscriptAttributeName range:range]; }


//! Generated by NSAttributedStringCategoryImplementation.py
- (void)addAttributeBackgroundColor:(NSColor *)backgroundColor range:(NSRange)range { [self addAttribute:NSBackgroundColorAttributeName value:backgroundColor range:range]; }
//! Generated by NSAttributedStringCategoryImplementation.py
- (void)removeAttributeBackgroundColorFromRange:(NSRange)range { [self removeAttribute:NSBackgroundColorAttributeName range:range]; }


//! Generated by NSAttributedStringCategoryImplementation.py
- (void)addAttributeAttachment:(NSTextAttachment *)attachment range:(NSRange)range { [self addAttribute:NSAttachmentAttributeName value:attachment range:range]; }
//! Generated by NSAttributedStringCategoryImplementation.py
- (void)removeAttributeAttachmentFromRange:(NSRange)range { [self removeAttribute:NSAttachmentAttributeName range:range]; }


//! Generated by NSAttributedStringCategoryImplementation.py
- (void)addAttributeLigature:(NSLigatureType)ligature range:(NSRange)range { [self addAttribute:NSLigatureAttributeName value:[NSNumber numberWithUnsignedInteger:ligature] range:range]; }
//! Generated by NSAttributedStringCategoryImplementation.py
- (void)removeAttributeLigatureFromRange:(NSRange)range { [self removeAttribute:NSLigatureAttributeName range:range]; }


//! Generated by NSAttributedStringCategoryImplementation.py
- (void)addAttributeBaselineOffset:(float)baselineOffset range:(NSRange)range { [self addAttribute:NSBaselineOffsetAttributeName value:[NSNumber numberWithFloat:baselineOffset] range:range]; }
//! Generated by NSAttributedStringCategoryImplementation.py
- (void)removeAttributeBaselineOffsetFromRange:(NSRange)range { [self removeAttribute:NSBaselineOffsetAttributeName range:range]; }


//! Generated by NSAttributedStringCategoryImplementation.py
- (void)addAttributeKern:(float)kern range:(NSRange)range { [self addAttribute:NSKernAttributeName value:[NSNumber numberWithFloat:kern] range:range]; }
//! Generated by NSAttributedStringCategoryImplementation.py
- (void)removeAttributeKernFromRange:(NSRange)range { [self removeAttribute:NSKernAttributeName range:range]; }


//! Generated by NSAttributedStringCategoryImplementation.py
- (void)addAttributeLink:(NSURL *)link range:(NSRange)range { [self addAttribute:NSLinkAttributeName value:link range:range]; }
//! Generated by NSAttributedStringCategoryImplementation.py
- (void)removeAttributeLinkFromRange:(NSRange)range { [self removeAttribute:NSLinkAttributeName range:range]; }


//! Generated by NSAttributedStringCategoryImplementation.py
- (void)addAttributeStrokeWidth:(float)strokeWidth range:(NSRange)range { [self addAttribute:NSStrokeWidthAttributeName value:[NSNumber numberWithFloat:strokeWidth] range:range]; }
//! Generated by NSAttributedStringCategoryImplementation.py
- (void)removeAttributeStrokeWidthFromRange:(NSRange)range { [self removeAttribute:NSStrokeWidthAttributeName range:range]; }


//! Generated by NSAttributedStringCategoryImplementation.py
- (void)addAttributeStrokeColor:(NSColor *)strokeColor range:(NSRange)range { [self addAttribute:NSStrokeColorAttributeName value:strokeColor range:range]; }
//! Generated by NSAttributedStringCategoryImplementation.py
- (void)removeAttributeStrokeColorFromRange:(NSRange)range { [self removeAttribute:NSStrokeColorAttributeName range:range]; }


//! Generated by NSAttributedStringCategoryImplementation.py
- (void)addAttributeUnderlineColor:(NSColor *)underlineColor range:(NSRange)range { [self addAttribute:NSUnderlineColorAttributeName value:underlineColor range:range]; }
//! Generated by NSAttributedStringCategoryImplementation.py
- (void)removeAttributeUnderlineColorFromRange:(NSRange)range { [self removeAttribute:NSUnderlineColorAttributeName range:range]; }


//! Generated by NSAttributedStringCategoryImplementation.py
- (void)addAttributeStrikethroughStyle:(NSUnderlineStyle)strikethroughStyle range:(NSRange)range { [self addAttribute:NSStrikethroughStyleAttributeName value:[NSNumber numberWithUnsignedInteger:strikethroughStyle] range:range]; }
//! Generated by NSAttributedStringCategoryImplementation.py
- (void)removeAttributeStrikethroughStyleFromRange:(NSRange)range { [self removeAttribute:NSStrikethroughStyleAttributeName range:range]; }


//! Generated by NSAttributedStringCategoryImplementation.py
- (void)addAttributeStrikethroughColor:(NSColor *)strikethroughColor range:(NSRange)range { [self addAttribute:NSStrikethroughColorAttributeName value:strikethroughColor range:range]; }
//! Generated by NSAttributedStringCategoryImplementation.py
- (void)removeAttributeStrikethroughColorFromRange:(NSRange)range { [self removeAttribute:NSStrikethroughColorAttributeName range:range]; }


//! Generated by NSAttributedStringCategoryImplementation.py
- (void)addAttributeShadow:(NSShadow *)shadow range:(NSRange)range { [self addAttribute:NSShadowAttributeName value:shadow range:range]; }
//! Generated by NSAttributedStringCategoryImplementation.py
- (void)removeAttributeShadowFromRange:(NSRange)range { [self removeAttribute:NSShadowAttributeName range:range]; }


//! Generated by NSAttributedStringCategoryImplementation.py
- (void)addAttributeObliqueness:(float)obliqueness range:(NSRange)range { [self addAttribute:NSObliquenessAttributeName value:[NSNumber numberWithFloat:obliqueness] range:range]; }
//! Generated by NSAttributedStringCategoryImplementation.py
- (void)removeAttributeObliquenessFromRange:(NSRange)range { [self removeAttribute:NSObliquenessAttributeName range:range]; }


//! Generated by NSAttributedStringCategoryImplementation.py
- (void)addAttributeExpansion:(float)expansion range:(NSRange)range { [self addAttribute:NSExpansionAttributeName value:[NSNumber numberWithFloat:expansion] range:range]; }
//! Generated by NSAttributedStringCategoryImplementation.py
- (void)removeAttributeExpansionFromRange:(NSRange)range { [self removeAttribute:NSExpansionAttributeName range:range]; }


//! Generated by NSAttributedStringCategoryImplementation.py
- (void)addAttributeCursor:(NSCursor *)cursor range:(NSRange)range { [self addAttribute:NSCursorAttributeName value:cursor range:range]; }
//! Generated by NSAttributedStringCategoryImplementation.py
- (void)removeAttributeCursorFromRange:(NSRange)range { [self removeAttribute:NSCursorAttributeName range:range]; }


//! Generated by NSAttributedStringCategoryImplementation.py
- (void)addAttributeToolTip:(NSString *)toolTip range:(NSRange)range { [self addAttribute:NSToolTipAttributeName value:toolTip range:range]; }
//! Generated by NSAttributedStringCategoryImplementation.py
- (void)removeAttributeToolTipFromRange:(NSRange)range { [self removeAttribute:NSToolTipAttributeName range:range]; }

#if __MAC_OS_X_VERSION_MIN_REQUIRED >= 1050
//! Generated by NSAttributedStringCategoryImplementation.py
- (void)addAttributeMarkedClauseSegment:(NSInteger)markedClauseSegment range:(NSRange)range { [self addAttribute:NSMarkedClauseSegmentAttributeName value:[NSNumber numberWithInteger:markedClauseSegment] range:range]; }
//! Generated by NSAttributedStringCategoryImplementation.py
- (void)removeAttributeMarkedClauseSegmentFromRange:(NSRange)range { [self removeAttribute:NSMarkedClauseSegmentAttributeName range:range]; }
#endif
#if __MAC_OS_X_VERSION_MIN_REQUIRED >= 1070
//! Generated by NSAttributedStringCategoryImplementation.py
- (void)addAttributeVerticalGlyphForm:(NSVerticalGlyphForm)verticalGlyphForm range:(NSRange)range { [self addAttribute:NSVerticalGlyphFormAttributeName value:[NSNumber numberWithUnsignedInteger:verticalGlyphForm] range:range]; }
//! Generated by NSAttributedStringCategoryImplementation.py
- (void)removeAttributeVerticalGlyphFormFromRange:(NSRange)range { [self removeAttribute:NSVerticalGlyphFormAttributeName range:range]; }
#endif
#if __MAC_OS_X_VERSION_MIN_REQUIRED >= 1080
//! Generated by NSAttributedStringCategoryImplementation.py
- (void)addAttributeTextAlternatives:(NSTextAlternatives *)textAlternatives range:(NSRange)range { [self addAttribute:NSTextAlternativesAttributeName value:textAlternatives range:range]; }
//! Generated by NSAttributedStringCategoryImplementation.py
- (void)removeAttributeTextAlternativesFromRange:(NSRange)range { [self removeAttribute:NSTextAlternativesAttributeName range:range]; }
#endif
@end


@implementation NSAttributedStringAttributeDictionary

- (void)setObject:(id)anObject forKey:(id<NSCopying>)aKey {
    if (anObject == nil) {
        [super removeObjectForKey:aKey];
    } else {
        [super setObject:anObject forKey:aKey];
    }
}


- (NSLigatureType)ligature {
    return self.ligatureNumber ? [self.ligatureNumber unsignedIntegerValue] : NSLigatureStandard;
}

- (void)setLigature:(NSLigatureType)ligature {
    self.ligatureNumber = [NSNumber numberWithUnsignedInteger:ligature];
}

- (NSNumber *)writingDirectionArray {
    return [self objectForKey:NSWritingDirectionAttributeName];
}

- (void)setWritingDirectionArray:(NSArray *)writingDirectionArray {
    [self setObject:writingDirectionArray forKey:NSWritingDirectionAttributeName];
}

- (NSWritingDirectionAttributes)writingDirection {
    NSArray *array = self.writingDirectionArray;
    NSWritingDirectionAttributes attr = 0;
    for (NSNumber *value in array) {
        attr |= 1 << [value unsignedIntegerValue];
    }
    return attr;
}

-(void)setWritingDirection:(NSWritingDirectionAttributes)writingDirection {
    self.writingDirectionArray = __NSWritingDirectionAttributeArrayFromAttributes(writingDirection);
}



//! Generated by NSAttributedStringDictionaryImplementation.py
- (NSFont *)font { return [self objectForKey:NSFontAttributeName]; }
//! Generated by NSAttributedStringDictionaryImplementation.py
- (void)setFont:(NSFont *)font { [self setObject:font forKey:NSFontAttributeName]; }

//! Generated by NSAttributedStringDictionaryImplementation.py
- (NSParagraphStyle *)paragraphStyle { return [self objectForKey:NSParagraphStyleAttributeName]; }
//! Generated by NSAttributedStringDictionaryImplementation.py
- (void)setParagraphStyle:(NSParagraphStyle *)paragraphStyle { [self setObject:paragraphStyle forKey:NSParagraphStyleAttributeName]; }

//! Generated by NSAttributedStringDictionaryImplementation.py
- (NSColor *)foregroundColor { return [self objectForKey:NSForegroundColorAttributeName]; }
//! Generated by NSAttributedStringDictionaryImplementation.py
- (void)setForegroundColor:(NSColor *)foregroundColor { [self setObject:foregroundColor forKey:NSForegroundColorAttributeName]; }

//! Generated by NSAttributedStringDictionaryImplementation.py
- (NSNumber *)underlineStyleNumber { return [self objectForKey:NSUnderlineStyleAttributeName]; }
//! Generated by NSAttributedStringDictionaryImplementation.py
- (void)setUnderlineStyleNumber:(NSNumber *)underlineStyleNumber { [self setObject:underlineStyleNumber forKey:NSUnderlineStyleAttributeName]; }

//! Generated by NSAttributedStringDictionaryImplementation.py
- (NSUnderlineStyle)underlineStyle { return [self.underlineStyleNumber unsignedIntegerValue]; }
//! Generated by NSAttributedStringDictionaryImplementation.py
- (void)setUnderlineStyle:(NSUnderlineStyle)underlineStyle { self.underlineStyleNumber = [NSNumber numberWithUnsignedInteger:underlineStyle]; }

//! Generated by NSAttributedStringDictionaryImplementation.py
- (NSNumber *)superscriptNumber { return [self objectForKey:NSSuperscriptAttributeName]; }
//! Generated by NSAttributedStringDictionaryImplementation.py
- (void)setSuperscriptNumber:(NSNumber *)superscriptNumber { [self setObject:superscriptNumber forKey:NSSuperscriptAttributeName]; }

//! Generated by NSAttributedStringDictionaryImplementation.py
- (BOOL)superscript { return [self.superscriptNumber boolValue]; }
//! Generated by NSAttributedStringDictionaryImplementation.py
- (void)setSuperscript:(BOOL)superscript { self.superscriptNumber = [NSNumber numberWithBool:superscript]; }

//! Generated by NSAttributedStringDictionaryImplementation.py
- (NSColor *)backgroundColor { return [self objectForKey:NSBackgroundColorAttributeName]; }
//! Generated by NSAttributedStringDictionaryImplementation.py
- (void)setBackgroundColor:(NSColor *)backgroundColor { [self setObject:backgroundColor forKey:NSBackgroundColorAttributeName]; }

//! Generated by NSAttributedStringDictionaryImplementation.py
- (NSTextAttachment *)attachment { return [self objectForKey:NSAttachmentAttributeName]; }
//! Generated by NSAttributedStringDictionaryImplementation.py
- (void)setAttachment:(NSTextAttachment *)attachment { [self setObject:attachment forKey:NSAttachmentAttributeName]; }

//! Generated by NSAttributedStringDictionaryImplementation.py
- (NSNumber *)ligatureNumber { return [self objectForKey:NSLigatureAttributeName]; }
//! Generated by NSAttributedStringDictionaryImplementation.py
- (void)setLigatureNumber:(NSNumber *)ligatureNumber { [self setObject:ligatureNumber forKey:NSLigatureAttributeName]; }

//! Generated by NSAttributedStringDictionaryImplementation.py
- (NSNumber *)baselineOffsetNumber { return [self objectForKey:NSBaselineOffsetAttributeName]; }
//! Generated by NSAttributedStringDictionaryImplementation.py
- (void)setBaselineOffsetNumber:(NSNumber *)baselineOffsetNumber { [self setObject:baselineOffsetNumber forKey:NSBaselineOffsetAttributeName]; }

//! Generated by NSAttributedStringDictionaryImplementation.py
- (float)baselineOffset { return [self.baselineOffsetNumber floatValue]; }
//! Generated by NSAttributedStringDictionaryImplementation.py
- (void)setBaselineOffset:(float)baselineOffset { self.baselineOffsetNumber = [NSNumber numberWithFloat:baselineOffset]; }

//! Generated by NSAttributedStringDictionaryImplementation.py
- (NSNumber *)kernNumber { return [self objectForKey:NSKernAttributeName]; }
//! Generated by NSAttributedStringDictionaryImplementation.py
- (void)setKernNumber:(NSNumber *)kernNumber { [self setObject:kernNumber forKey:NSKernAttributeName]; }

//! Generated by NSAttributedStringDictionaryImplementation.py
- (float)kern { return [self.kernNumber floatValue]; }
//! Generated by NSAttributedStringDictionaryImplementation.py
- (void)setKern:(float)kern { self.kernNumber = [NSNumber numberWithFloat:kern]; }

//! Generated by NSAttributedStringDictionaryImplementation.py
- (NSURL *)link { return [self objectForKey:NSLinkAttributeName]; }
//! Generated by NSAttributedStringDictionaryImplementation.py
- (void)setLink:(NSURL *)link { [self setObject:link forKey:NSLinkAttributeName]; }

//! Generated by NSAttributedStringDictionaryImplementation.py
- (NSNumber *)strokeWidthNumber { return [self objectForKey:NSStrokeWidthAttributeName]; }
//! Generated by NSAttributedStringDictionaryImplementation.py
- (void)setStrokeWidthNumber:(NSNumber *)strokeWidthNumber { [self setObject:strokeWidthNumber forKey:NSStrokeWidthAttributeName]; }

//! Generated by NSAttributedStringDictionaryImplementation.py
- (float)strokeWidth { return [self.strokeWidthNumber floatValue]; }
//! Generated by NSAttributedStringDictionaryImplementation.py
- (void)setStrokeWidth:(float)strokeWidth { self.strokeWidthNumber = [NSNumber numberWithFloat:strokeWidth]; }

//! Generated by NSAttributedStringDictionaryImplementation.py
- (NSColor *)strokeColor { return [self objectForKey:NSStrokeColorAttributeName]; }
//! Generated by NSAttributedStringDictionaryImplementation.py
- (void)setStrokeColor:(NSColor *)strokeColor { [self setObject:strokeColor forKey:NSStrokeColorAttributeName]; }

//! Generated by NSAttributedStringDictionaryImplementation.py
- (NSColor *)underlineColor { return [self objectForKey:NSUnderlineColorAttributeName]; }
//! Generated by NSAttributedStringDictionaryImplementation.py
- (void)setUnderlineColor:(NSColor *)underlineColor { [self setObject:underlineColor forKey:NSUnderlineColorAttributeName]; }

//! Generated by NSAttributedStringDictionaryImplementation.py
- (NSNumber *)strikethroughStyleNumber { return [self objectForKey:NSStrikethroughStyleAttributeName]; }
//! Generated by NSAttributedStringDictionaryImplementation.py
- (void)setStrikethroughStyleNumber:(NSNumber *)strikethroughStyleNumber { [self setObject:strikethroughStyleNumber forKey:NSStrikethroughStyleAttributeName]; }

//! Generated by NSAttributedStringDictionaryImplementation.py
- (NSUnderlineStyle)strikethroughStyle { return [self.strikethroughStyleNumber unsignedIntegerValue]; }
//! Generated by NSAttributedStringDictionaryImplementation.py
- (void)setStrikethroughStyle:(NSUnderlineStyle)strikethroughStyle { self.strikethroughStyleNumber = [NSNumber numberWithUnsignedInteger:strikethroughStyle]; }

//! Generated by NSAttributedStringDictionaryImplementation.py
- (NSColor *)strikethroughColor { return [self objectForKey:NSStrikethroughColorAttributeName]; }
//! Generated by NSAttributedStringDictionaryImplementation.py
- (void)setStrikethroughColor:(NSColor *)strikethroughColor { [self setObject:strikethroughColor forKey:NSStrikethroughColorAttributeName]; }

//! Generated by NSAttributedStringDictionaryImplementation.py
- (NSShadow *)shadow { return [self objectForKey:NSShadowAttributeName]; }
//! Generated by NSAttributedStringDictionaryImplementation.py
- (void)setShadow:(NSShadow *)shadow { [self setObject:shadow forKey:NSShadowAttributeName]; }

//! Generated by NSAttributedStringDictionaryImplementation.py
- (NSNumber *)obliquenessNumber { return [self objectForKey:NSObliquenessAttributeName]; }
//! Generated by NSAttributedStringDictionaryImplementation.py
- (void)setObliquenessNumber:(NSNumber *)obliquenessNumber { [self setObject:obliquenessNumber forKey:NSObliquenessAttributeName]; }

//! Generated by NSAttributedStringDictionaryImplementation.py
- (float)obliqueness { return [self.obliquenessNumber floatValue]; }
//! Generated by NSAttributedStringDictionaryImplementation.py
- (void)setObliqueness:(float)obliqueness { self.obliquenessNumber = [NSNumber numberWithFloat:obliqueness]; }

//! Generated by NSAttributedStringDictionaryImplementation.py
- (NSNumber *)expansionNumber { return [self objectForKey:NSExpansionAttributeName]; }
//! Generated by NSAttributedStringDictionaryImplementation.py
- (void)setExpansionNumber:(NSNumber *)expansionNumber { [self setObject:expansionNumber forKey:NSExpansionAttributeName]; }

//! Generated by NSAttributedStringDictionaryImplementation.py
- (float)expansion { return [self.expansionNumber floatValue]; }
//! Generated by NSAttributedStringDictionaryImplementation.py
- (void)setExpansion:(float)expansion { self.expansionNumber = [NSNumber numberWithFloat:expansion]; }

//! Generated by NSAttributedStringDictionaryImplementation.py
- (NSCursor *)cursor { return [self objectForKey:NSCursorAttributeName]; }
//! Generated by NSAttributedStringDictionaryImplementation.py
- (void)setCursor:(NSCursor *)cursor { [self setObject:cursor forKey:NSCursorAttributeName]; }

//! Generated by NSAttributedStringDictionaryImplementation.py
- (NSString *)toolTip { return [self objectForKey:NSToolTipAttributeName]; }
//! Generated by NSAttributedStringDictionaryImplementation.py
- (void)setToolTip:(NSString *)toolTip { [self setObject:toolTip forKey:NSToolTipAttributeName]; }

#if __MAC_OS_X_VERSION_MIN_REQUIRED >= 1050
//! Generated by NSAttributedStringDictionaryImplementation.py
- (NSNumber *)markedClauseSegmentNumber { return [self objectForKey:NSMarkedClauseSegmentAttributeName]; }
//! Generated by NSAttributedStringDictionaryImplementation.py
- (void)setMarkedClauseSegmentNumber:(NSNumber *)markedClauseSegmentNumber { [self setObject:markedClauseSegmentNumber forKey:NSMarkedClauseSegmentAttributeName]; }
#endif
#if __MAC_OS_X_VERSION_MIN_REQUIRED >= 1050
//! Generated by NSAttributedStringDictionaryImplementation.py
- (NSInteger)markedClauseSegment { return [self.markedClauseSegmentNumber integerValue]; }
//! Generated by NSAttributedStringDictionaryImplementation.py
- (void)setMarkedClauseSegment:(NSInteger)markedClauseSegment { self.markedClauseSegmentNumber = [NSNumber numberWithInteger:markedClauseSegment]; }
#endif
#if __MAC_OS_X_VERSION_MIN_REQUIRED >= 1070
//! Generated by NSAttributedStringDictionaryImplementation.py
- (NSNumber *)verticalGlyphFormNumber { return [self objectForKey:NSVerticalGlyphFormAttributeName]; }
//! Generated by NSAttributedStringDictionaryImplementation.py
- (void)setVerticalGlyphFormNumber:(NSNumber *)verticalGlyphFormNumber { [self setObject:verticalGlyphFormNumber forKey:NSVerticalGlyphFormAttributeName]; }
#endif
#if __MAC_OS_X_VERSION_MIN_REQUIRED >= 1070
//! Generated by NSAttributedStringDictionaryImplementation.py
- (NSVerticalGlyphForm)verticalGlyphForm { return [self.verticalGlyphFormNumber unsignedIntegerValue]; }
//! Generated by NSAttributedStringDictionaryImplementation.py
- (void)setVerticalGlyphForm:(NSVerticalGlyphForm)verticalGlyphForm { self.verticalGlyphFormNumber = [NSNumber numberWithUnsignedInteger:verticalGlyphForm]; }
#endif
#if __MAC_OS_X_VERSION_MIN_REQUIRED >= 1080
//! Generated by NSAttributedStringDictionaryImplementation.py
- (NSTextAlternatives *)textAlternatives { return [self objectForKey:NSTextAlternativesAttributeName]; }
//! Generated by NSAttributedStringDictionaryImplementation.py
- (void)setTextAlternatives:(NSTextAlternatives *)textAlternatives { [self setObject:textAlternatives forKey:NSTextAlternativesAttributeName]; }
#endif
@end

